from pait.app.any import (
    Pait,
    SimpleRoute,
    add_multi_simple_route,
    add_simple_route,
    get_app_attribute,
    http_exception,
    load_app,
    load_class_app,
    pait,
    set_app_attribute,
)
